@@ -77,36 +77,44 @@ postModeOrtho <- function(y, x, priorCoef=momprior(tau=0.348), priorDelta=modelb
       #Refine grid for phi by adding further promising models
       goodmodelsizes <- which((maxpp - pp[-1]) < log(1000))
       if (length(goodmodelsizes)>0) {
-          goodmodelsizes <- 1:(goodmodelsizes[length(goodmodelsizes)])
-          goodmodelsizes <- goodmodelsizes[goodmodelsizes <= maxvars]
-          phiseqextra <- modelidextra <- modelusumextra <- ppextra <- vector("list",length(goodmodelsizes))
-          for (i in 1:length(goodmodelsizes)) {
-              nvars <- goodmodelsizes[i]
-              maxmodels <- lchoose(ncol(x),nvars)
-              if (maxmodels>0) {
-                  if (nvars == 1) {
-                      #idx <- list(2,3,4)
-                      idx <- list(2,3,4,5,6)
-                  } else {
-                      if (nvars>2) idx <- 1:(nvars-2) else idx <- integer(0)
-                      #idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars-1,nvars+2), c(idx,nvars,nvars+1))
-                      idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars-1,nvars+2), c(idx,nvars,nvars+1), c(idx,nvars-1,nvars+3), c(idx,nvars,nvars+2))
+          if (length(goodmodelsizes)!=1 | goodmodelsizes[1]!=ncol(x)) {  #skip if only 1 model with ncol(x) variables remains
+              #goodmodelsizes <- 1:(goodmodelsizes[length(goodmodelsizes)])
+              goodmodelsizes <- goodmodelsizes[goodmodelsizes <= min(maxvars,ncol(x)-1)]
+              phiseqextra <- modelidextra <- modelusumextra <- ppextra <- vector("list",length(goodmodelsizes))
+              for (i in 1:length(goodmodelsizes)) {
+                  nvars <- goodmodelsizes[i]
+                  maxmodels <- lchoose(ncol(x),nvars)
+                  if (maxmodels>0) {
+                      if (nvars == 1) {
+                          idx <- list(2,3,4,5,6) #idx <- list(2,3,4)
+                          idx <- idx[idx<=ncol(x)]
+                      } else {
+                          if (nvars>2) idx <- 1:(nvars-2) else idx <- integer(0)
+                          if (nvars+3<=ncol(x)) {
+                              idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars-1,nvars+2), c(idx,nvars,nvars+1), c(idx,nvars-1,nvars+3), c(idx,nvars,nvars+2))
+                          } else if (nvars+2<=ncol(x)) {
+                              idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars-1,nvars+2), c(idx,nvars,nvars+1))
+                          } else {
+                              idx <- list(c(idx,nvars-1,nvars+1), c(idx,nvars,nvars+1))
+                          }
+                      }
+                      idx <- idx[1:min(length(idx),ifelse(maxmodels>=log(6),5,exp(maxmodels)-1))]  #if <5 extra models available, just take those
+                      modelidextra[[i]] <- sapply(idx,function(z) paste(o[z],collapse=','))
+                      modelusumextra[[i]] <- sapply(idx, function(z) sum(uord[z]))
+                      lposextra <- sumy2 - shrinkage * modelusumextra[[i]]
+                      phiseqextra[[i]] <- .5* lposextra / (.5*apos + 1)
+                      ppextra[[i]] <- priorModel(nvars) - 0.5*nvars*log(1+g) - 0.5*apos * log(lposextra)
                   }
-                  idx <- idx[1:ifelse(maxmodels>=log(6),5,exp(maxmodels)-1)]  #if <5 extra models available, just take those
-                  modelidextra[[i]] <- sapply(idx,function(z) paste(o[z],collapse=','))
-                  modelusumextra[[i]] <- sapply(idx, function(z) sum(uord[z]))
-                  lposextra <- sumy2 - shrinkage * modelusumextra[[i]]
-                  phiseqextra[[i]] <- .5* lposextra / (.5*apos + 1)
-                  ppextra[[i]] <- priorModel(nvars) - 0.5*nvars*log(1+g) - 0.5*apos * log(lposextra)
               }
+              phiseq <- c(phiseq,unlist(phiseqextra))
+              pp <- c(pp,unlist(ppextra))
+              modelid <- c(modelid,unlist(modelidextra))
+              modelusum <- c(modelusum,unlist(modelusumextra))
+              #variableidsextra <- strsplit(as.character(modelidextra),split=',')
+              variableidsextra <- strsplit(unlist(modelidextra),split=',')
+              variableids <- c(variableids,variableidsextra)
+              nvars <- c(nvars,sapply(variableidsextra,length))
           }
-          phiseq <- c(phiseq,unlist(phiseqextra))
-          pp <- c(pp,unlist(ppextra))
-          modelid <- c(modelid,unlist(modelidextra))
-          modelusum <- c(modelusum,unlist(modelusumextra))
-          variableidsextra <- strsplit(as.character(modelidextra),split=',')
-          variableids <- c(variableids,variableidsextra)
-          nvars <- c(nvars,sapply(variableidsextra,length))
       }
       #Evaluate marginal of phi at selected grid points
       phiseq <- phiseq[order(phiseq,decreasing=TRUE)]
@@ -115,6 +123,13 @@ postModeOrtho <- function(y, x, priorCoef=momprior(tau=0.348), priorDelta=modelb
       } else {  #pMOM
           phiseqpost <- jointPhiyMOMOrtho(phiseq, sumy2, apos, shrinkage, u, g, rho, logscale=TRUE)
       }
+      #Avoid phi values associated to 0 posterior density
+      zeropost <- match(-Inf,phiseqpost)
+      if (!is.na(zeropost)) {
+          zerophi <- phiseq[zeropost]
+          phiseq <- phiseq[1:max(1,zeropost-1)]
+          phiseqpost <- phiseqpost[1:max(1,zeropost-1)]
+      }
       #Refine grid where target increases > tolf % its max value
       maxphipost <- max(phiseqpost)
       tolf <- 0.01
@@ -181,6 +196,10 @@ postModeOrtho <- function(y, x, priorCoef=momprior(tau=0.348), priorDelta=modelb
     #
     models <- data.frame(modelid=modelid,pp=pp)
     models <- models[order(pp,decreasing=TRUE),]
+    if (sum(models$pp)>1) {
+        warning("Posterior model probabilities are unstable, this often signals that t(x) %*% x is not diagonal")
+        models$pp <- models$pp/sum(models$pp)
+    }
     ans <- list(models=models, phi=phi, logpy=logpy)
     if (bma) {
         margpp <- double(p)
@@ -282,6 +301,13 @@ postModeBlockDiag <- function(y, x, blocks, priorCoef=zellnerprior(tau=nrow(x)),
         names(umv[[i]]) <- modelid[-1]
         ubest[[i]] <- do.call(rbind,by(u[[i]], INDICES=u[[i]][,'nvars'], FUN=function(z) z[1,]))[-1,] #best model of each size
     }
+    #Avoid overflow in cases where X'X not exactly block-diagonal
+    sumumax <- sum(sapply(u, function(z) z$u[1]))
+    if (sumumax > (sumy2-l.phi)) {
+        u <- lapply(u, function(z) { z$u <- z$u * ((sumy2-l.phi)/sumumax); return(z) } )
+        ubest <- lapply(ubest, function(z) { z$u <- z$u * ((sumy2-l.phi)/sumumax); return(z) } )
+        for (i in 1:length(umv)) { umv[[i]] <- lapply(umv[[i]], function(z) { z$u <- z$u * ((sumy2-l.phi)/sumumax); return(z) } ) }
+    }
     #Enumerate models
     models <- coolblock(ubest,g=g,priorModelBlock=priorModelBlock,varidx=varidx,maxvars=maxvars)
     models <- rbind(data.frame(nvars=0,u=0,modelid='',phi=Inf,m=NA,block=NA,u.lower=0,u.upper=0,stringsAsFactors=FALSE),models)
@@ -345,7 +371,8 @@ postModeBlockDiag <- function(y, x, blocks, priorCoef=zellnerprior(tau=nrow(x)),
     #Refine grid where target increases > tolf % its max value
     tolf <- 0.01
     sel <- which(abs(diff(exp(phiseqpost-maxphipost))) > tolf)
-    while (length(sel)>0) {
+    itrefine <- 0
+    while ((length(sel)>0) && (itrefine<10)) {
         phiseqnew <- unlist(lapply(sel, function(z) { ans= seq(phiseq[z],phiseq[z+1],length=5); ans[c(-1,-length(ans))] } ))
         if (priorCoef@priorDistr == 'zellner') {
             phiseqpostnew <- jointPhiyZellnerBlockDiag(phiseqnew, sumy2, apos, a.phi, l.phi, shrinkage, u, g, priorModelBlock, blocksize, logscale=TRUE)
@@ -358,11 +385,14 @@ postModeBlockDiag <- function(y, x, blocks, priorCoef=zellnerprior(tau=nrow(x)),
         ophiseq <- order(phiseq,decreasing=TRUE)
         phiseq <- phiseq[ophiseq]; phiseqpost <- phiseqpost[ophiseq]
         sel <- which(abs(diff(exp(phiseqpost-maxphipost))) > tolf)
+        itrefine <- itrefine+1
     }
     #Compute marginal p(y)
+    zeropost <- which(is.infinite(phiseqpost))
+    if (length(zeropost)>1) { phiseq <- phiseq[-zeropost[-1]]; phiseqpost <- phiseqpost[-zeropost[-1]] }
     phiseqpost <- exp(phiseqpost-maxphipost)
     if (integrateMethod=='CSR') {
-        myintegral <- int.simpson2(phiseq,phiseqpost,equi=FALSE,method="CSR")
+        myintegral <- int.simpson2(phiseq[phiseqpost!=0],phiseqpost[phiseqpost!=0],equi=FALSE,method="CSR")
     } else {
         if (priorCoef@priorDistr=='zellner') {
             f2int <- function(phi) { exp(jointPhiyZellnerBlockDiag(phi,sumy2=sumy2,apos=apos,a.phi=a.phi,l.phi=l.phi,shrinkage=shrinkage,u=u,g=g,priorModelBlock=priorModelBlock,blocksize=blocksize,logscale=TRUE) - maxphipost) }
@@ -475,21 +505,25 @@ coolblock <- function(ubest, g, priorModelBlock, varidx, maxvars) {
         r[[k]] <- matrix(NA,nrow=nvars[k]+1,ncol=nvars[k]+1)
         rownames(r[[k]]) <- colnames(r[[k]]) <- 0:nvars[k]
         r[[k]][-1,1] <- r[[k]][1,-1] <- shrinkage * ubest[[k]][,'u'] / (ubest[[k]]$nvars * log1g - 2 * sapply(priorodds,'[[',1))
-        for (l in 2:nvars[k]) {
-            for (j in 1:(l-1)) {
-                r[[k]][l+1,j+1] <- r[[k]][j+1,l+1] <- shrinkage * (ubest[[k]][l,'u'] - ubest[[k]][j,'u']) / ((l-j) * log1g - 2 * priorodds[[l]][j+1])
-            }
+        if (nvars[k]>=2) {
+          for (l in 2:nvars[k]) {
+              for (j in 1:(l-1)) {
+                  r[[k]][l+1,j+1] <- r[[k]][j+1,l+1] <- shrinkage * (ubest[[k]][l,'u'] - ubest[[k]][j,'u']) / ((l-j) * log1g - 2 * priorodds[[l]][j+1])
+              }
+          }
         }
         phiseq[[k]] <- double(nvars[k])
         m[[k]] <- character(nvars[k])
         ll <- 1
-        for (l in 1:(nvars[k]-1)) {
-            rmin <- min(r[[k]][l+1,1:l])
-            rmax <- max(r[[k]][(l+2):(nvars[k]+1),l+1])
-            if (rmax < rmin) {
-                phiseq[[k]][ll] <- rmin
-                m[[k]][ll] <- as.character(ubest[[k]][l,'modelid'])
-                ll <- ll+1
+        if (nvars[k]>=2) {
+            for (l in 1:(nvars[k]-1)) {
+                rmin <- min(r[[k]][l+1,1:l])
+                rmax <- max(r[[k]][(l+2):(nvars[k]+1),l+1])
+                if (rmax < rmin) {
+                    phiseq[[k]][ll] <- rmin
+                    m[[k]][ll] <- as.character(ubest[[k]][l,'modelid'])
+                    ll <- ll+1
+                }
             }
         }
         l <- nvars[k]  #treat last case separately (no larger models to compare with)
